package com.jiuqi.syntax;

import java.util.Date;
import java.lang.reflect.InvocationTargetException;
import java.util.Calendar;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Hashtable;
import java.util.Map;

public final class AutoFunctionListener extends CustomFunctionListener
{
    private Object provider;
    private String classify;
    private Map funcMap;
    private static final int SY_FUNC_CALL_ERROR = 10400;
    private static final int SY_FUNC_RET_ERROR = 10401;
    private static final int SY_FUNC_ACL_ERROR = 10402;
    
    public AutoFunctionListener(final Object provider) {
        this(provider, null);
    }
    
    public AutoFunctionListener(final Object provider, final String classify) {
        this.provider = provider;
        this.classify = classify;
        this.initFunctions();
    }
    
    private void initFunctions() {
        this.funcMap = new Hashtable();
        if (this.provider == null) {
            return;
        }
        final Class pc = this.provider.getClass();
        final Method[] methods = pc.getDeclaredMethods();
    Label_0239:
        for (int i = 0; i < methods.length; ++i) {
            if (Modifier.isPublic(methods[i].getModifiers())) {
                final Function func = new Function();
                func.name = methods[i].getName();
                func.resultType = this.convertType(methods[i].getReturnType());
                if (func.resultType != 0) {
                    func.classify = this.classify;
                    final Class[] paramTypes = methods[i].getParameterTypes();
                    for (int j = 0; j < paramTypes.length; ++j) {
                        final FunctionParam param = new FunctionParam();
                        param.name = "p" + Integer.toString(j);
                        param.type = this.convertType(paramTypes[j]);
                        if (param.type == 0) {
                            continue Label_0239;
                        }
                        param.canOmit = false;
                        param.isSign = false;
                        func.params.add(param);
                    }
                    this.functions.add(func);
                    this.funcMap.put(func, methods[i]);
                }
            }
        }
    }
    
    private int convertType(final Class typeClass) {
        if (Integer.TYPE.equals(typeClass) || Integer.class.equals(typeClass)) {
            return 1;
        }
        if (Double.TYPE.equals(typeClass) || Double.class.equals(typeClass)) {
            return 2;
        }
        if (String.class.equals(typeClass)) {
            return 3;
        }
        if (Boolean.TYPE.equals(typeClass) || Boolean.class.equals(typeClass)) {
            return 5;
        }
        if (Calendar.class.equals(typeClass)) {
            return 6;
        }
        return 0;
    }
    
    public String getClassify() {
        return this.classify;
    }
    
    @Override
    public int callFunction(final String funcName, final CommonDatas paramValues, final CommonData resultData, final boolean isCheckMode) {
        final Function func = this.functions.getFunc(funcName);
        if (func == null) {
            return 201;
        }
        final Method method = this.funcMap.get(func);
        try {
            final Object[] args = this.convertParams(paramValues, func);
            final Object ret = method.invoke(this.provider, args);
            resultData.type = func.resultType;
            if (!this.convertObject(ret, resultData)) {
                return 10401;
            }
        }
        catch (final SyntaxException e) {
            return e.getErrorCode();
        }
        catch (final IllegalArgumentException ex) {
            return 202;
        }
        catch (final IllegalAccessException ex2) {
            return 10402;
        }
        catch (final InvocationTargetException e2) {
            e2.printStackTrace();
            return 10400;
        }
        return 0;
    }
    
    private Object[] convertParams(final CommonDatas paramValues, final Function func) throws SyntaxException {
        final Object[] params = new Object[func.params.size()];
        for (int i = 0; i < func.params.size(); ++i) {
            final CommonData value = (paramValues.size() <= i) ? null : paramValues.getItem(i);
            if (value == null || value.isNull) {
                params[i] = null;
            }
            else {
                switch (func.params.getParam(i).type) {
                    case 1: {
                        params[i] = new Integer((int)value.getIntRealValue());
                        break;
                    }
                    case 2: {
                        params[i] = new Double(value.getIntRealValue());
                        break;
                    }
                    case 3: {
                        params[i] = value.cdString;
                        break;
                    }
                    case 5: {
                        params[i] = new Boolean(value.cdBool);
                        break;
                    }
                    case 6: {
                        params[i] = value.cdDate;
                        break;
                    }
                    default: {
                        throw new SyntaxException(202);
                    }
                }
            }
        }
        return params;
    }
    
    private boolean convertObject(final Object value, final CommonData result) {
        if (value == null) {
            result.isNull = true;
            result.initNullValue();
        }
        else {
            result.isNull = false;
            switch (result.type) {
                case 1: {
                    if (value instanceof Integer) {
                        result.cdInt = (int)value;
                        break;
                    }
                    if (value instanceof Double) {
                        result.cdInt = ((Double)value).intValue();
                        break;
                    }
                    return false;
                }
                case 2:
                    Label_0153: {
                        if (value instanceof Integer) {
                            result.cdReal = (int)value;
                            break Label_0153;
                        }
                        if (value instanceof Double) {
                            result.cdReal = (double)value;
                            break Label_0153;
                        }
                        return false;
                    }
                case 3: {
                    result.cdString = value.toString();
                    break;
                }
                case 5: {
                    if (value instanceof Boolean) {
                        result.cdBool = (boolean)value;
                        break;
                    }
                    return false;
                }
                case 6: {
                    if (value instanceof Calendar) {
                        result.cdDate = (Calendar)value;
                        break;
                    }
                    if (value instanceof Date) {
                        result.setDateValue((Date)value);
                        break;
                    }
                    if (value instanceof Long) {
                        final Calendar date = Calendar.getInstance();
                        date.setTimeInMillis((long)value);
                        result.cdDate = date;
                        break;
                    }
                    return false;
                }
                default: {
                    return false;
                }
            }
        }
        return true;
    }
    
    @Override
    public String getCodeMessage(final int retCode) {
        switch (retCode) {
            case 10400: {
                return "函数调用时出现程序异常";
            }
            case 10401: {
                return "错误的函数返回值类型";
            }
            case 10402: {
                return "类或函数无法访问";
            }
            default: {
                return null;
            }
        }
    }
    
    private class SyntaxException extends Exception
    {
        private int errorCode;
        
        public SyntaxException(final int errorCode) {
            this.errorCode = errorCode;
        }
        
        public int getErrorCode() {
            return this.errorCode;
        }
    }
}
